(*
 *  bserv, Internet Relay Chat services, bircrandom.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  some modifications by peter green
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
unit bircrandom;

{
delphi's random function has a behavior which makes it unuseable for the nospoof ping:
the numbers are completely predictable.

i now use something based on MD5, i looked at ircu random code.
}

interface

{the seed is now actually a string, not a number, so it can also be things like a phrase}
procedure setircrandomseed(const s:string);
procedure feedrandomness(const data;size:integer);
function ircrandom:integer;
procedure time64bits(var result);

implementation

uses
  {$ifdef linux}
  linux,
  {$else}
  windows,
  {$endif}
  fastmd5,btime;

var
  state:array[0..3] of integer;

procedure time64bits(var result);
var
{$ifdef linux}
  v:ttimeval absolute result;
begin
  gettimeofday(v);
{$else}
  v:tlargeinteger absolute result;
begin
  queryperformancecounter(v);
{$endif}
end;


procedure feedrandomness(const data;size:integer);
var
  buf:array[0..63] of byte;
  i:array[0..3] of integer;
  a:integer;
begin
  if size > 56 then size := 56;
  if size < 0 then size := 0;
  move(data,buf[8],size);
  move(state[0],buf[0],8);
  i[0] := $67452301;
  i[1] := $EFCDAB89;
  i[2] := $98BADCFE;
  i[3] := $10325476;
  md5processblock(i,buf);
  for a := 0 to 3 do state[a] := state[a] xor i[a];
end;

procedure setircrandomseed(const s:string);
begin
  feedrandomness(s[1],length(s));
end;

function ircrandom:integer;
var
  i:packed record
    t:array[0..1] of integer;
    d:double;
  end;
begin
  time64bits(i.t);
  i.d := unixtimefloat;
  feedrandomness(i,sizeof(i));
  result := state[2] xor state[3];
end;

end.
